@@ -73,7 +73,10 @@ class IPCLock(locking.Lock):
         else:
             return True
 
-    def acquire(self, blocking=True):
+    def acquire(self, blocking=True, shared=False):
+        if shared:
+            raise tooz.NotImplemented
+
         if (blocking is not True and
                 sysv_ipc.SEMAPHORE_TIMEOUT_SUPPORTED is False):
             raise tooz.NotImplemented("This system does not support"
@@ -85,21 +88,23 @@ class IPCLock(locking.Lock):
         elif blocking and timeout is not None:
             start_time = time.time()
         while True:
+            tmplock = None
             try:
-                self._lock = sysv_ipc.Semaphore(self.key,
-                                                flags=sysv_ipc.IPC_CREX,
-                                                initial_value=1)
-                self._lock.undo = True
+                tmplock = sysv_ipc.Semaphore(self.key,
+                                             flags=sysv_ipc.IPC_CREX,
+                                             initial_value=1)
+                tmplock.undo = True
             except sysv_ipc.ExistentialError:
                 # We failed to create it because it already exists, then try to
                 # grab the existing one.
                 try:
-                    self._lock = sysv_ipc.Semaphore(self.key)
-                    self._lock.undo = True
+                    tmplock = sysv_ipc.Semaphore(self.key)
+                    tmplock.undo = True
                 except sysv_ipc.ExistentialError:
                     # Semaphore has been deleted in the mean time, retry from
                     # the beginning!
                     continue
+
             if start_time is not None:
                 elapsed = max(0.0, time.time() - start_time)
                 if elapsed >= timeout:
@@ -109,24 +114,24 @@ class IPCLock(locking.Lock):
             else:
                 adjusted_timeout = timeout
             try:
-                self._lock.acquire(timeout=adjusted_timeout)
+                tmplock.acquire(timeout=adjusted_timeout)
             except sysv_ipc.BusyError:
-                self._lock = None
+                tmplock = None
                 return False
             except sysv_ipc.ExistentialError:
                 # Likely the lock has been deleted in the meantime, retry
                 continue
             else:
+                self._lock = tmplock
                 return True
 
     def release(self):
         if self._lock is not None:
             try:
                 self._lock.remove()
+                self._lock = None
             except sysv_ipc.ExistentialError:
                 return False
-            finally:
-                self._lock = None
             return True
         return False
 
@@ -148,6 +153,7 @@ class IPCDriver(coordination.CoordinationDriver):
     """
 
     CHARACTERISTICS = (
+        coordination.Characteristics.NON_TIMEOUT_BASED,
         coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
         coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
     )
@@ -163,7 +169,7 @@ class IPCDriver(coordination.CoordinationDriver):
 
     def __init__(self, member_id, parsed_url, options):
         """Initialize the IPC driver."""
-        super(IPCDriver, self).__init__()
+        super(IPCDriver, self).__init__(member_id)
         self._executor = utils.ProxyExecutor.build("IPC", options)
 
     def _start(self):
@@ -193,7 +199,7 @@ class IPCDriver(coordination.CoordinationDriver):
     def _write_group_list(self, group_list):
         data = msgpack.dumps(list(group_list))
         if len(data) >= self._SEGMENT_SIZE - 2:
-            raise coordination.ToozError("Group list is too big")
+            raise tooz.ToozError("Group list is too big")
         self._group_list.write(struct.pack('H', len(data)))
         self._group_list.write(data, offset=2)
 
@@ -219,6 +225,16 @@ class IPCDriver(coordination.CoordinationDriver):
 
         return IPCFutureResult(self._executor.submit(_delete_group))
 
+    def watch_join_group(self, group_id, callback):
+        # Check the group exist
+        self.get_members(group_id).get()
+        super(IPCDriver, self).watch_join_group(group_id, callback)
+
+    def watch_leave_group(self, group_id, callback):
+        # Check the group exist
+        self.get_members(group_id).get()
+        super(IPCDriver, self).watch_leave_group(group_id, callback)
+
     def _get_groups_handler(self):
         with self._lock:
             return self._read_group_list()
@@ -231,30 +247,6 @@ class IPCDriver(coordination.CoordinationDriver):
     def get_lock(name):
         return IPCLock(name)
 
-    @staticmethod
-    def watch_join_group(group_id, callback):
-        raise tooz.NotImplemented
-
-    @staticmethod
-    def unwatch_join_group(group_id, callback):
-        raise tooz.NotImplemented
-
-    @staticmethod
-    def watch_leave_group(group_id, callback):
-        raise tooz.NotImplemented
-
-    @staticmethod
-    def unwatch_leave_group(group_id, callback):
-        raise tooz.NotImplemented
-
-    @staticmethod
-    def watch_elected_as_leader(group_id, callback):
-        raise tooz.NotImplemented
-
-    @staticmethod
-    def unwatch_elected_as_leader(group_id, callback):
-        raise tooz.NotImplemented
-
 
 class IPCFutureResult(coordination.CoordAsyncResult):
     """IPC asynchronous result that references a future."""
@@ -265,9 +257,9 @@ class IPCFutureResult(coordination.CoordAsyncResult):
         try:
             return self._fut.result(timeout=timeout)
         except futures.TimeoutError as e:
-            coordination.raise_with_cause(coordination.OperationTimedOut,
-                                          encodeutils.exception_to_unicode(e),
-                                          cause=e)
+            utils.raise_with_cause(coordination.OperationTimedOut,
+                                   encodeutils.exception_to_unicode(e),
+                                   cause=e)
 
     def done(self):
         return self._fut.done()
